@@ -9,6 +9,7 @@
 #include "models/IARURegions.hpp"
 #include "Audio/AudioDevice.hpp"
 #include "Transceiver/Transceiver.hpp"
+#include "otpgenerator.h"
 
 #include "pimpl_h.hpp"
 
@@ -137,6 +138,7 @@ public:
   bool decode_at_52s () const;
   bool single_decode () const;
   bool twoPass() const;
+  bool superFox() const;
   bool bFox() const;
   bool bHound() const;
   bool bLowSidelobes() const;
@@ -183,15 +185,22 @@ public:
   bool highlight_only_fields () const;
   bool include_WAE_entities () const;
   bool highlight_73 () const;
+  void setSpecial_Q65_Pileup();
   void setSpecial_Hound();
   void setSpecial_Fox();
   void setSpecial_None();
+  void toggle_SF();
   bool highlight_DXcall () const;
   bool highlight_DXgrid () const;
   bool Individual_Contest_Name() const;
-
-//                                      0       1      2         3       4       5      6     7        8
-  enum class SpecialOperatingActivity {NONE, NA_VHF, EU_VHF, FIELD_DAY, RTTY, WW_DIGI, FOX, HOUND, ARRL_DIGI};
+  bool validate_otp_seed(QString);
+  QString OTPSeed() const;
+  QString OTPUrl() const;
+  bool OTPEnabled() const;
+  bool ShowOTP() const;
+  unsigned int OTPinterval() const;
+//                                      0       1      2         3       4       5      6     7        8           9
+  enum class SpecialOperatingActivity {NONE, NA_VHF, EU_VHF, FIELD_DAY, RTTY, WW_DIGI, FOX, HOUND, ARRL_DIGI, Q65_PILEUP};
   SpecialOperatingActivity special_op_id () const;
 
   struct CalibrationParams
@@ -243,6 +252,8 @@ public:
   // Close down connection to rig.
   void transceiver_offline ();
 
+  void set_CTY_DAT_version(QString const& version);
+
   // Set transceiver frequency in Hertz.
   Q_SLOT void transceiver_frequency (Frequency);
 
