@@ -6,16 +6,16 @@
 #include <limits>
 
 #include <QUdpSocket>
+#include <QNetworkInterface>
 #include <QHostInfo>
 #include <QTimer>
 #include <QQueue>
 #include <QByteArray>
-#include <QHostAddress>
 #include <QColor>
 #include <QDebug>
 
 #include "NetworkMessage.hpp"
-
+#include "qt_helpers.hpp"
 #include "pimpl_impl.hpp"
 
 #include "moc_MessageClient.cpp"
@@ -34,13 +34,15 @@ class MessageClient::impl
 
 public:
   impl (QString const& id, QString const& version, QString const& revision,
-        port_type server_port, MessageClient * self)
+        port_type server_port, int TTL, MessageClient * self)
     : self_ {self}
     , enabled_ {false}
     , id_ {id}
     , version_ {version}
     , revision_ {revision}
+    , dns_lookup_id_ {-1}
     , server_port_ {server_port}
+    , TTL_ {TTL}
     , schema_ {2}  // use 2 prior to negotiation not 1 which is broken
     , heartbeat_timer_ {new QTimer {this}}
   {
@@ -48,46 +50,50 @@ public:
     connect (this, &QIODevice::readyRead, this, &impl::pending_datagrams);
 
     heartbeat_timer_->start (NetworkMessage::pulse * 1000);
-
-    // bind to an ephemeral port
-    bind ();
   }
 
   ~impl ()
   {
     closedown ();
+    if (dns_lookup_id_ != -1)
+      {
+        QHostInfo::abortHostLookup (dns_lookup_id_);
+      }
   }
 
   enum StreamStatus {Fail, Short, OK};
 
-  void parse_message (QByteArray const& msg);
+  void set_server (QString const& server_name, QStringList const& network_interface_names);
+  Q_SLOT void host_info_results (QHostInfo);
+  void start ();
+  void parse_message (QByteArray const&);
   void pending_datagrams ();
   void heartbeat ();
   void closedown ();
   StreamStatus check_status (QDataStream const&) const;
-  void send_message (QByteArray const&);
-  void send_message (QDataStream const& out, QByteArray const& message)
+  void send_message (QByteArray const&, bool queue_if_pending = true, bool allow_duplicates = false);
+  void send_message (QDataStream const& out, QByteArray const& message, bool queue_if_pending = true, bool allow_duplicates = false)
   {
-      if (OK == check_status (out))
-        {
-          send_message (message);
-        }
-      else
-        {
-          Q_EMIT self_->error ("Error creating UDP message");
-        }
+    if (OK == check_status (out))
+      {
+        send_message (message, queue_if_pending, allow_duplicates);
+      }
+    else
+      {
+        Q_EMIT self_->error ("Error creating UDP message");
+      }
   }
 
-  Q_SLOT void host_info_results (QHostInfo);
-
   MessageClient * self_;
   bool enabled_;
   QString id_;
   QString version_;
   QString revision_;
-  QString server_string_;
-  port_type server_port_;
+  int dns_lookup_id_;
   QHostAddress server_;
+  port_type server_port_;
+  int TTL_;
+  std::vector<QNetworkInterface> network_interfaces_;
   quint32 schema_;
   QTimer * heartbeat_timer_;
   std::vector<QHostAddress> blocked_addresses_;
@@ -99,35 +105,100 @@ public:
 
 #include "MessageClient.moc"
 
+void MessageClient::impl::set_server (QString const& server_name, QStringList const& network_interface_names)
+{
+  // qDebug () << "MessageClient server:" << server_name << "port:" << server_port_ << "interfaces:" << network_interface_names;
+  server_.setAddress (server_name);
+  network_interfaces_.clear ();
+  for (auto const& net_if_name : network_interface_names)
+    {
+      network_interfaces_.push_back (QNetworkInterface::interfaceFromName (net_if_name));
+    }
+
+  if (server_.isNull () && server_name.size ()) // DNS lookup required
+    {
+      // queue a host address lookup
+#if QT_VERSION >= QT_VERSION_CHECK(5, 9, 0)
+      dns_lookup_id_ = QHostInfo::lookupHost (server_name, this, &MessageClient::impl::host_info_results);
+#else
+      dns_lookup_id_ = QHostInfo::lookupHost (server_name, this, SLOT (host_info_results (QHostInfo)));
+#endif
+    }
+  else
+    {
+      start ();
+    }
+}
+
 void MessageClient::impl::host_info_results (QHostInfo host_info)
 {
+  if (host_info.lookupId () != dns_lookup_id_) return;
+  dns_lookup_id_ = -1;
   if (QHostInfo::NoError != host_info.error ())
     {
-      Q_EMIT self_->error ("UDP server lookup failed:\n" + host_info.errorString ());
-      pending_messages_.clear (); // discard
+      Q_EMIT self_->error ("UDP server DNS lookup failed: " + host_info.errorString ());
+      return;
     }
-  else if (host_info.addresses ().size ())
+  else
     {
-      auto server = host_info.addresses ()[0];
-      if (blocked_addresses_.end () == std::find (blocked_addresses_.begin (), blocked_addresses_.end (), server))
+      auto const& server_addresses = host_info.addresses ();
+      if (server_addresses.size ())
         {
-          server_ = server;
-          TRACE_UDP ("resulting server:" << server);
-
-          // send initial heartbeat which allows schema negotiation
-          heartbeat ();
-
-          // clear any backlog
-          while (pending_messages_.size ())
-            {
-              send_message (pending_messages_.dequeue ());
-            }
+          server_ = server_addresses[0];
         }
-      else
+    }
+  start ();
+}
+
+void MessageClient::impl::start ()
+{
+  if (server_.isNull ())
+    {
+      Q_EMIT self_->close ();
+      pending_messages_.clear (); // discard
+      return;
+    }
+
+  if (is_broadcast_address (server_))
+    {
+      Q_EMIT self_->error ("IPv4 broadcast not supported, please specify the loop-back address, a server host address, or multicast group address");
+      pending_messages_.clear (); // discard
+      return;
+    }
+
+  if (blocked_addresses_.end () != std::find (blocked_addresses_.begin (), blocked_addresses_.end (), server_))
+    {
+      Q_EMIT self_->error ("UDP server blocked, please try another");
+      pending_messages_.clear (); // discard
+      return;
+    }
+
+  TRACE_UDP ("Trying server:" << server_.toString ());
+  QHostAddress interface_addr {IPv6Protocol == server_.protocol () ? QHostAddress::AnyIPv6 : QHostAddress::AnyIPv4};
+
+  if (localAddress () != interface_addr)
+    {
+      if (UnconnectedState != state () || state ())
         {
-          Q_EMIT self_->error ("UDP server blocked, please try another");
-          pending_messages_.clear (); // discard
+          close ();
         }
+      // bind to an ephemeral port on the selected interface and set
+      // up for sending datagrams
+      bind (interface_addr);
+      // qDebug () << "Bound to UDP port:" << localPort () << "on:" << localAddress ();
+
+      // set multicast TTL to limit scope when sending to multicast
+      // group addresses
+      setSocketOption (MulticastTtlOption, TTL_);
+    }
+
+  // send initial heartbeat which allows schema negotiation
+  heartbeat ();
+
+  // clear any backlog
+  while (pending_messages_.size ())
+    {
+      send_message (pending_messages_.dequeue (), true, false);
     }
 }
 
@@ -187,7 +258,13 @@ void MessageClient::impl::parse_message (QByteArray const& msg)
                 quint8 modifiers {0};
                 in >> time >> snr >> delta_time >> delta_frequency >> mode >> message
                    >> low_confidence >> modifiers;
-                TRACE_UDP ("Reply: time:" << time << "snr:" << snr << "dt:" << delta_time << "df:" << delta_frequency << "mode:" << mode << "message:" << message << "low confidence:" << low_confidence << "modifiers: 0x" << hex << modifiers);
+                TRACE_UDP ("Reply: time:" << time << "snr:" << snr << "dt:" << delta_time << "df:" << delta_frequency << "mode:" << mode << "message:" << message << "low confidence:" << low_confidence << "modifiers: 0x"
+#if QT_VERSION >= QT_VERSION_CHECK (5, 15, 0)
+                           << Qt::hex
+#else
+                           << hex
+#endif
+                           << modifiers);
                 if (check_status (in) != Fail)
                   {
                     Q_EMIT self_->reply (time, snr, delta_time, delta_frequency
@@ -315,7 +392,22 @@ void MessageClient::impl::parse_message (QByteArray const& msg)
               }
               break;
 
-            default:
+              case NetworkMessage::AnnotationInfo: {
+                QByteArray dx_call;
+                bool sort_order_provided{false};
+                quint32 sort_order{std::numeric_limits<quint32>::max()};
+                in >> dx_call >> sort_order_provided >> sort_order;
+                TRACE_UDP ("External Callsign Info:" << dx_call << "sort_order_provided:" << sort_order_provided
+                                                     << "sort_order:" << sort_order);
+                if (sort_order > 50000) sort_order = 50000;
+                if (check_status(in) != Fail) {
+                  Q_EMIT
+                  self_->annotation_info(QString::fromUtf8(dx_call), sort_order_provided, sort_order);
+                }
+              }
+                break;
+
+              default:
               // Ignore
               //
               // Note that although server  heartbeat messages are not
@@ -349,14 +441,11 @@ void MessageClient::impl::heartbeat ()
    if (server_port_ && !server_.isNull ())
     {
       QByteArray message;
-      NetworkMessage::Builder hb {&message, NetworkMessage::Heartbeat, id_, schema_};
-      hb << NetworkMessage::Builder::schema_number // maximum schema number accepted
-         << version_.toUtf8 () << revision_.toUtf8 ();
-      if (OK == check_status (hb))
-        {
-          TRACE_UDP ("schema:" << schema_ << "max schema:" << NetworkMessage::Builder::schema_number << "version:" << version_ << "revision:" << revision_);
-          writeDatagram (message, server_, server_port_);
-        }
+      NetworkMessage::Builder out {&message, NetworkMessage::Heartbeat, id_, schema_};
+      out << NetworkMessage::Builder::schema_number // maximum schema number accepted
+          << version_.toUtf8 () << revision_.toUtf8 ();
+      TRACE_UDP ("schema:" << schema_ << "max schema:" << NetworkMessage::Builder::schema_number << "version:" << version_ << "revision:" << revision_);
+      send_message (out, message, false, true);
     }
 }
 
@@ -366,27 +455,38 @@ void MessageClient::impl::closedown ()
     {
       QByteArray message;
       NetworkMessage::Builder out {&message, NetworkMessage::Close, id_, schema_};
-      if (OK == check_status (out))
-        {
-          TRACE_UDP ("");
-          writeDatagram (message, server_, server_port_);
-        }
+      TRACE_UDP ("");
+      send_message (out, message, false);
     }
 }
 
-void MessageClient::impl::send_message (QByteArray const& message)
+void MessageClient::impl::send_message (QByteArray const& message, bool queue_if_pending, bool allow_duplicates)
 {
   if (server_port_)
     {
       if (!server_.isNull ())
         {
-          if (message != last_message_) // avoid duplicates
+          if (allow_duplicates || message != last_message_) // avoid duplicates
             {
-              writeDatagram (message, server_, server_port_);
+              if (is_multicast_address (server_))
+                {
+                  // send datagram on each selected network interface
+                  std::for_each (network_interfaces_.begin (), network_interfaces_.end ()
+                                 , [&] (QNetworkInterface const& net_if) {
+                                     setMulticastInterface (net_if);
+                                     // qDebug () << "Multicast UDP datagram sent to:" << server_ << "port:" << server_port_ << "on:" << multicastInterface ().humanReadableName ();
+                                     writeDatagram (message, server_, server_port_);
+                                   });
+                }
+              else
+                {
+                  // qDebug () << "Unicast UDP datagram sent to:" << server_ << "port:" << server_port_;
+                  writeDatagram (message, server_, server_port_);
+                }
               last_message_ = message;
             }
         }
-      else
+      else if (queue_if_pending)
         {
           pending_messages_.enqueue (message);
         }
@@ -419,35 +519,31 @@ auto MessageClient::impl::check_status (QDataStream const& stream) const -> Stre
 }
 
 MessageClient::MessageClient (QString const& id, QString const& version, QString const& revision,
-                              QString const& server, port_type server_port, QObject * self)
+                              QString const& server_name, port_type server_port,
+                              QStringList const& network_interface_names,
+                              int TTL, QObject * self)
   : QObject {self}
-  , m_ {id, version, revision, server_port, this}
+  , m_ {id, version, revision, server_port, TTL, this}
 {
+  connect (&*m_
 #if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
-  connect (&*m_, static_cast<void (impl::*) (impl::SocketError)> (&impl::error)
-           , [this] (impl::SocketError e)
-           {
+           , static_cast<void (impl::*) (impl::SocketError)> (&impl::error), [this] (impl::SocketError e)
+#else
+           , &impl::errorOccurred, [this] (impl::SocketError e)
+#endif
+                                   {
 #if defined (Q_OS_WIN)
-             if (e != impl::NetworkError // take this out when Qt 5.5
-                                         // stops doing this
-                                         // spuriously
-                 && e != impl::ConnectionRefusedError) // not
-                                                       // interested
-                                                       // in this with
-                                                       // UDP socket
+                                     if (e != impl::NetworkError // take this out when Qt 5.5 stops doing this spuriously
+                                         && e != impl::ConnectionRefusedError) // not interested in this with UDP socket
+                                       {
 #else
-             Q_UNUSED (e);
+                                       {
+                                         Q_UNUSED (e);
 #endif
-               {
-                 Q_EMIT error (m_->errorString ());
-               }
-           });
-#else
-  connect (&*m_, &impl::errorOccurred, [this] (impl::SocketError) {
                                          Q_EMIT error (m_->errorString ());
+                                       }
                                        });
-#endif
-  set_server (server);
+  m_->set_server (server_name, network_interface_names);
 }
 
 QHostAddress MessageClient::server_address () const
@@ -460,16 +556,9 @@ auto MessageClient::server_port () const -> port_type
   return m_->server_port_;
 }
 
-void MessageClient::set_server (QString const& server)
+void MessageClient::set_server (QString const& server_name, QStringList const& network_interface_names)
 {
-  m_->server_.clear ();
-  m_->server_string_ = server;
-  if (!server.isEmpty ())
-    {
-      // queue a host address lookup
-      TRACE_UDP ("server host DNS lookup:" << server);
-      QHostInfo::lookupHost (server, &*m_, SLOT (host_info_results (QHostInfo)));
-    }
+  m_->set_server (server_name, network_interface_names);
 }
 
 void MessageClient::set_server_port (port_type server_port)
@@ -477,25 +566,10 @@ void MessageClient::set_server_port (port_type server_port)
   m_->server_port_ = server_port;
 }
 
-qint64 MessageClient::send_raw_datagram (QByteArray const& message, QHostAddress const& dest_address
-                                       , port_type dest_port)
+void MessageClient::set_TTL (int TTL)
 {
-  if (dest_port && !dest_address.isNull ())
-    {
-      return m_->writeDatagram (message, dest_address, dest_port);
-    }
-  return 0;
-}
-
-void MessageClient::add_blocked_destination (QHostAddress const& a)
-{
-  m_->blocked_addresses_.push_back (a);
-  if (a == m_->server_)
-    {
-      m_->server_.clear ();
-      Q_EMIT error ("UDP server blocked, please try another");
-      m_->pending_messages_.clear (); // discard
-    }
+  m_->TTL_ = TTL;
+  m_->setSocketOption (QAbstractSocket::MulticastTtlOption, m_->TTL_);
 }
 
 void MessageClient::enable (bool flag)
@@ -511,17 +585,19 @@ void MessageClient::status_update (Frequency f, QString const& mode, QString con
                                    , bool watchdog_timeout, QString const& sub_mode
                                    , bool fast_mode, quint8 special_op_mode
                                    , quint32 frequency_tolerance, quint32 tr_period
-                                   , QString const& configuration_name)
+                                   , QString const& configuration_name
+                                   , QString const& tx_message)
 {
-  if (m_->server_port_ && !m_->server_string_.isEmpty ())
+  if (m_->server_port_ && !m_->server_.isNull ())
     {
       QByteArray message;
       NetworkMessage::Builder out {&message, NetworkMessage::Status, m_->id_, m_->schema_};
       out << f << mode.toUtf8 () << dx_call.toUtf8 () << report.toUtf8 () << tx_mode.toUtf8 ()
           << tx_enabled << transmitting << decoding << rx_df << tx_df << de_call.toUtf8 ()
           << de_grid.toUtf8 () << dx_grid.toUtf8 () << watchdog_timeout << sub_mode.toUtf8 ()
-          << fast_mode << special_op_mode << frequency_tolerance << tr_period << configuration_name.toUtf8 ();
-      TRACE_UDP ("frequency:" << f << "mode:" << mode << "DX:" << dx_call << "report:" << report << "Tx mode:" << tx_mode << "tx_enabled:" << tx_enabled << "Tx:" << transmitting << "decoding:" << decoding << "Rx df:" << rx_df << "Tx df:" << tx_df << "DE:" << de_call << "DE grid:" << de_grid << "DX grid:" << dx_grid << "w/d t/o:" << watchdog_timeout << "sub_mode:" << sub_mode << "fast mode:" << fast_mode << "spec op mode:" << special_op_mode << "frequency tolerance:" << frequency_tolerance << "T/R period:" << tr_period << "configuration name:" << configuration_name);
+          << fast_mode << special_op_mode << frequency_tolerance << tr_period << configuration_name.toUtf8 ()
+          << tx_message.toUtf8 ();
+      TRACE_UDP ("frequency:" << f << "mode:" << mode << "DX:" << dx_call << "report:" << report << "Tx mode:" << tx_mode << "tx_enabled:" << tx_enabled << "Tx:" << transmitting << "decoding:" << decoding << "Rx df:" << rx_df << "Tx df:" << tx_df << "DE:" << de_call << "DE grid:" << de_grid << "DX grid:" << dx_grid << "w/d t/o:" << watchdog_timeout << "sub_mode:" << sub_mode << "fast mode:" << fast_mode << "spec op mode:" << special_op_mode << "frequency tolerance:" << frequency_tolerance << "T/R period:" << tr_period << "configuration name:" << configuration_name << "Tx message:" << tx_message);
       m_->send_message (out, message);
     }
 }
@@ -530,7 +606,7 @@ void MessageClient::decode (bool is_new, QTime time, qint32 snr, float delta_tim
                             , QString const& mode, QString const& message_text, bool low_confidence
                             , bool off_air)
 {
-   if (m_->server_port_ && !m_->server_string_.isEmpty ())
+   if (m_->server_port_ && !m_->server_.isNull ())
     {
       QByteArray message;
       NetworkMessage::Builder out {&message, NetworkMessage::Decode, m_->id_, m_->schema_};
@@ -545,7 +621,7 @@ void MessageClient::WSPR_decode (bool is_new, QTime time, qint32 snr, float delt
                                  , qint32 drift, QString const& callsign, QString const& grid, qint32 power
                                  , bool off_air)
 {
-   if (m_->server_port_ && !m_->server_string_.isEmpty ())
+   if (m_->server_port_ && !m_->server_.isNull ())
     {
       QByteArray message;
       NetworkMessage::Builder out {&message, NetworkMessage::WSPRDecode, m_->id_, m_->schema_};
@@ -558,7 +634,7 @@ void MessageClient::WSPR_decode (bool is_new, QTime time, qint32 snr, float delt
 
 void MessageClient::decodes_cleared ()
 {
-   if (m_->server_port_ && !m_->server_string_.isEmpty ())
+   if (m_->server_port_ && !m_->server_.isNull ())
     {
       QByteArray message;
       NetworkMessage::Builder out {&message, NetworkMessage::Clear, m_->id_, m_->schema_};
@@ -573,24 +649,24 @@ void MessageClient::qso_logged (QDateTime time_off, QString const& dx_call, QStr
                                 , QString const& comments, QString const& name, QDateTime time_on
                                 , QString const& operator_call, QString const& my_call
                                 , QString const& my_grid, QString const& exchange_sent
-                                , QString const& exchange_rcvd)
+                                , QString const& exchange_rcvd, QString const& propmode)
 {
-   if (m_->server_port_ && !m_->server_string_.isEmpty ())
+   if (m_->server_port_ && !m_->server_.isNull ())
     {
       QByteArray message;
       NetworkMessage::Builder out {&message, NetworkMessage::QSOLogged, m_->id_, m_->schema_};
       out << time_off << dx_call.toUtf8 () << dx_grid.toUtf8 () << dial_frequency << mode.toUtf8 ()
           << report_sent.toUtf8 () << report_received.toUtf8 () << tx_power.toUtf8 () << comments.toUtf8 ()
           << name.toUtf8 () << time_on << operator_call.toUtf8 () << my_call.toUtf8 () << my_grid.toUtf8 ()
-          << exchange_sent.toUtf8 () << exchange_rcvd.toUtf8 ();
-      TRACE_UDP ("time off:" << time_off << "DX:" << dx_call << "DX grid:" << dx_grid << "dial:" << dial_frequency << "mode:" << mode << "sent:" << report_sent << "rcvd:" << report_received << "pwr:" << tx_power << "comments:" << comments << "name:" << name << "time on:" << time_on << "op:" << operator_call << "DE:" << my_call << "DE grid:" << my_grid << "exch sent:" << exchange_sent << "exch rcvd:" << exchange_rcvd);
+          << exchange_sent.toUtf8 () << exchange_rcvd.toUtf8 () << propmode.toUtf8 ();
+      TRACE_UDP ("time off:" << time_off << "DX:" << dx_call << "DX grid:" << dx_grid << "dial:" << dial_frequency << "mode:" << mode << "sent:" << report_sent << "rcvd:" << report_received << "pwr:" << tx_power << "comments:" << comments << "name:" << name << "time on:" << time_on << "op:" << operator_call << "DE:" << my_call << "DE grid:" << my_grid << "exch sent:" << exchange_sent << "exch rcvd:" << exchange_rcvd  << "prop_mode:" << propmode);
       m_->send_message (out, message);
     }
 }
 
 void MessageClient::logged_ADIF (QByteArray const& ADIF_record)
 {
-   if (m_->server_port_ && !m_->server_string_.isEmpty ())
+   if (m_->server_port_ && !m_->server_.isNull ())
     {
       QByteArray message;
       NetworkMessage::Builder out {&message, NetworkMessage::LoggedADIF, m_->id_, m_->schema_};
