@@ -1,5 +1,4 @@
-subroutine sync8(dd,nfa,nfb,syncmin,nfqso,maxcand,nzhsym,candidate,   &
-     ncand,sbase)
+subroutine sync8(dd,npts,nfa,nfb,syncmin,nfqso,maxcand,candidate,ncand,sbase)
 
   include 'ft8_params.f90'
   parameter (MAXPRECAND=1000)
@@ -10,13 +9,13 @@ subroutine sync8(dd,nfa,nfb,syncmin,nfqso,maxcand,nzhsym,candidate,   &
   real s(NH1,NHSYM)
   real savg(NH1)
   real sbase(NH1)
-  real x(NFFT1)
+  real x(NFFT1+2)
   real sync2d(NH1,-JZ:JZ)
   real red(NH1)
   real red2(NH1)
   real candidate0(3,MAXPRECAND)
   real candidate(3,maxcand)
-  real dd(NMAX)
+  real dd(npts)
   integer jpeak(NH1)
   integer jpeak2(NH1)
   integer indx(NH1)
