@@ -167,7 +167,7 @@ void LogQSO::initLogQSO(QString const& hisCall, QString const& hisGrid, QString
   auto special_op = m_config->special_op_id ();
   if (SpOp::FOX == special_op
       || (m_config->autoLog ()
-          && SpOp::NONE < special_op && special_op < SpOp::FOX))
+          && ((SpOp::NONE < special_op && special_op < SpOp::FOX) || SpOp::ARRL_DIGI == special_op)))
     {
       // allow auto logging in Fox mode and contests
       accept();
@@ -221,7 +221,7 @@ void LogQSO::accept()
   }
 
   // validate
-  if (SpOp::NONE < special_op && special_op < SpOp::FOX)
+  if ((SpOp::NONE < special_op && special_op < SpOp::FOX) || (special_op > SpOp::HOUND))
     {
       if (xsent.isEmpty () || xrcvd.isEmpty ())
         {
