@@ -25,6 +25,7 @@
 #include <QQueue>
 #include <QFuture>
 #include <QFutureWatcher>
+#include <QDateTime>
 
 #include "MultiGeometryWidget.hpp"
 #include "NonInheritingProcess.hpp"
@@ -40,6 +41,9 @@
 #include "Network/PSKReporter.hpp"
 #include "logbook/logbook.h"
 #include "astro.h"
+#include "widgets/QSYMessageCreator.h"
+#include "widgets/QSYMessage.h"
+#include "widgets/qsymonitor.h"
 #include "MessageBox.hpp"
 #include "Network/NetworkAccessManager.hpp"
 
@@ -50,12 +54,14 @@
 #define NUM_MSK144_SYMBOLS 144             //s8 + d48 + s8 + d80
 #define NUM_Q65_SYMBOLS 85                 //63 data + 22 sync
 #define NUM_FT8_SYMBOLS 79
+#define NUM_SUPERFOX_SYMBOLS 153         //24 sync + 127 data + 2 ramp up/down
 #define NUM_FT4_SYMBOLS 105
 #define NUM_FST4_SYMBOLS 160             //240/2 data + 5*8 sync
 #define NUM_CW_SYMBOLS 250
 #define MAX_NUM_SYMBOLS 250
 #define TX_SAMPLE_RATE 48000
 #define NRING 3456000
+#define MAX_HOUNDS_IN_QUEUE 10
 
 extern int volatile itone[MAX_NUM_SYMBOLS];   //Audio tones for all Tx symbols
 extern int volatile icw[NUM_CW_SYMBOLS];	    //Dits for CW ID
@@ -141,9 +147,11 @@ private:
   void closeEvent(QCloseEvent *) override;
   void childEvent(QChildEvent *) override;
   bool eventFilter(QObject *, QEvent *) override;
+  void showQSYMessage(QString message);
 
 private slots:
   void initialize_fonts ();
+  void stopWRTimeout();
   void on_houndButton_clicked(bool checked);
   void on_ft8Button_clicked();
   void on_ft4Button_clicked();
@@ -164,9 +172,11 @@ private slots:
   void on_stopButton_clicked();
   void on_actionRelease_Notes_triggered ();
   void on_actionFT8_DXpedition_Mode_User_Guide_triggered();
+  void on_actionSuperFox_User_Guide_triggered();
   void on_actionQSG_FST4_triggered();
   void on_actionQSG_Q65_triggered();
   void on_actionQSG_X250_M3_triggered();
+  void on_actionQuick_Start_Guide_to_WSJT_X_2_7_and_QMAP_triggered();
   void on_actionOnline_User_Guide_triggered();
   void on_actionLocal_User_Guide_triggered();
   void on_actionWide_Waterfall_triggered();
@@ -211,6 +221,7 @@ private slots:
   void on_txb6_clicked();
   void on_lookupButton_clicked();
   void on_addButton_clicked();
+  void mousePressEvent(QMouseEvent *event) override;
   void on_dxCallEntry_textChanged (QString const&);
   void on_dxGridEntry_textChanged (QString const&);
   void on_dxCallEntry_editingFinished();
@@ -234,6 +245,7 @@ private slots:
   void on_reset_cabrillo_log_action_triggered ();
   void on_actionErase_wsjtx_log_adi_triggered();
   void on_actionErase_WSPR_hashtable_triggered();
+  void on_actionErase_list_of_Q65_callers_triggered();
   void on_actionExport_Cabrillo_log_triggered();
   void startTx2();
   void startP1();
@@ -262,6 +274,8 @@ private slots:
   void handle_transceiver_update (Transceiver::TransceiverState const&);
   void handle_transceiver_failure (QString const& reason);
   void on_actionAstronomical_data_toggled (bool);
+  void on_actionQSYMessage_Creator_triggered();
+  void on_actionQSY_Monitor_triggered();
   void on_actionShort_list_of_add_on_prefixes_and_suffixes_triggered();
   void band_changed (Frequency);
   void monitor (bool);
@@ -280,6 +294,7 @@ private slots:
   void on_actionAuto_Clear_Avg_toggled (bool);
   void VHF_features_enabled(bool b);
   void on_sbSubmode_valueChanged(int n);
+  void on_cbSendMsg_toggled(bool b);
   void on_cbShMsgs_toggled(bool b);
   void on_cbSWL_toggled(bool b);
   void on_cbTx6_toggled(bool b);
@@ -321,22 +336,28 @@ private slots:
   void on_measure_check_box_stateChanged (int);
   void on_sbNlist_valueChanged(int n);
   void on_sbNslots_valueChanged(int n);
-  void on_sbMax_dB_valueChanged(int n);
   void on_sbF_Low_valueChanged(int n);
   void on_sbF_High_valueChanged(int n);
   void chk_FST4_freq_range();
   void on_pbFoxReset_clicked();
+  void on_pbFreeText_clicked();
+  void FoxReset(QString reason);
   void on_comboBoxHoundSort_activated (int index);
   void not_GA_warning_message ();
   void checkMSK144ContestType();
   void on_pbBestSP_clicked();
   void on_RoundRobin_currentTextChanged(QString text);
-  void  setTxMsg(int n);
+  void setTxMsg(int n);
   bool stdCall(QString const& w);
   void remote_configure (QString const& mode, quint32 frequency_tolerance, QString const& submode
                          , bool fast_mode, quint32 tr_period, quint32 rx_df, QString const& dx_call
                          , QString const& dx_grid, bool generate_messages);
   void callSandP2(int nline);
+  void refreshHoundQueueDisplay();
+  void queueActiveWindowHound2(QString text);
+  void update_tx5(const QString &qsy_text);
+  void reply_tx5(const QString &qsy_text);
+  void setQSYMessageCreatorStatus(const bool &QSYMessageCreatorValue);
 
 private:
   Q_SIGNAL void initializeAudioOutputStream (QAudioDeviceInfo,
@@ -373,6 +394,9 @@ private:
   void setColorHighlighting();
   void chkFT4();
   bool elide_tx1_not_allowed () const;
+  void readWidebandDecodes();
+  void configActiveStations();
+  void sfox_tx();
 
   QProcessEnvironment const& m_env;
   NetworkAccessManager m_network_manager;
@@ -398,6 +422,9 @@ private:
   QScopedPointer<FastGraph> m_fastGraph;
   QScopedPointer<LogQSO> m_logDlg;
   QScopedPointer<Astro> m_astroWidget;
+  QScopedPointer<QSYMessageCreator> m_QSYMessageCreatorWidget;
+  QScopedPointer<QSYMessage> m_QSYMessageWidget;
+  QScopedPointer<QSYMonitor> m_qsymonitorWidget;
   QScopedPointer<HelpTextWindow> m_shortcuts;
   QScopedPointer<HelpTextWindow> m_prefixes;
   QScopedPointer<HelpTextWindow> m_mouseCmnds;
@@ -411,6 +438,8 @@ private:
   QString m_lastBand;
   QString m_lastCallsign;
   Frequency  m_dialFreqRxWSPR;  // best guess at WSPR QRG
+  bool m_QSYMessageCreatorValue = false;
+  bool m_qsymonitorValue = false;
 
   Detector * m_detector;
   unsigned m_FFTSize;
@@ -480,6 +509,7 @@ private:
   FrequencyList_v2_101::const_iterator m_frequency_list_fcal_iter;
   qint32  m_nTx73;
   qint32  m_UTCdisk;
+  QDateTime    m_UTCdiskDateTime;
   qint32  m_wait;
   qint32  m_isort;
   qint32  m_max_dB;
@@ -488,6 +518,7 @@ private:
   qint32  m_nHoundsCalling=0;
   qint32  m_Nlist=12;
   qint32  m_Nslots=5;
+  qint32  m_Nslots0=0;
   qint32  m_nFoxMsgTimes[5]={0,0,0,0,0};
   qint32  m_tAutoOn;
   qint32  m_tFoxTx=0;
@@ -508,6 +539,7 @@ private:
   qint32  m_score=0;
   qint32  m_fDop=0;
   qint32  m_echoSec0=0;
+  qint32  m_fetched=0;
 
   bool    m_btxok;		//True if OK to transmit
   bool    m_diskData;
@@ -525,7 +557,6 @@ private:
   bool    m_bDecoded;
   bool    m_noSuffix;
   bool    m_decodedText2;
-  bool    m_freeText;
   bool    m_sentFirst73;
   int     m_currentMessageType;
   QString m_currentMessage;
@@ -564,6 +595,7 @@ private:
   bool    m_bBestSPArmed=false;
   bool    m_bOK_to_chk=false;
   bool    m_bSentReport=false;
+  bool    m_discard_decoded_hounds_this_cycle=false;     // if something changes, like frequency, discard decoded messages that may be in-flight.
 
   SpecOp  m_specOp;
 
@@ -614,9 +646,13 @@ private:
   NonInheritingProcess p1;
   NonInheritingProcess p3;
 
+  QProcess p2;
+  QProcess p4;
+
   WSPRNet *wsprNet;
 
   QTimer m_guiTimer;
+  QTimer stopWRTimer;
   QTimer ptt1Timer;                 //StartTx delay
   QTimer ptt0Timer;                 //StopTx delay
   QTimer logQSOTimer;
@@ -664,7 +700,10 @@ private:
   QString m_BestCQpriority;
   QString m_deCall;
   QString m_deGrid;
+  QString m_freeTextMsg;
+  QString m_freeTextMsg0;
   QString m_ready2call[50];
+  QString m_callers[50];
 
   QSet<QString> m_pfx;
   QSet<QString> m_sfx;
@@ -682,6 +721,7 @@ private:
 
   QMap<QString,FoxQSO> m_foxQSO;       //Key = HoundCall, value = parameters for QSO in progress
   QMap<QString,QString> m_loggedByFox; //Key = HoundCall, value = logged band
+  QMap<QString,qint32> m_annotated_callsigns;  //Key = HoundCall, value = provided by api call
 
   struct FixupQSO       //Info for fixing Fox's log from file "FoxQSO.txt"
   {
@@ -701,6 +741,21 @@ private:
   };
   QMap<QString,ActiveCall> m_activeCall;   //Key = callsign, value = grid4, az, points for ARRL_DIGI
 
+  struct EMECall
+  {
+    QString grid4;
+    double frx;
+    double fsked;
+    qint32 nsnr;
+    qint32 t;
+    bool worked;
+    bool ready2call;
+    QString submode;
+  };
+  QMap<QString,EMECall> m_EMECall;
+
+  QMap<QString,bool> m_EMEworked;
+
   struct RecentCall
   {
     qint64 dialFreq;
@@ -722,7 +777,6 @@ private:
 
   QQueue<QString> m_houndQueue;        //Selected Hounds available for starting a QSO
   QQueue<QString> m_foxQSOinProgress;  //QSOs in progress: Fox has sent a report
-  QQueue<qint64>  m_foxRateQueue;
 
   QDateTime m_dateTimeQSOOn;
   QDateTime m_dateTimeLastTX;
@@ -835,6 +889,7 @@ private:
   QChar current_submode () const; // returns QChar {0} if submode is not appropriate
   void write_transmit_entry (QString const& file_name);
   void selectHound(QString t, bool bTopQueue);
+  void removeHoundFromCallingList(QString callsign);
   void houndCallers();
   void updateFoxQSOsInProgressDisplay();
   void foxQueueTopCallCommand();
@@ -842,12 +897,21 @@ private:
   void foxTxSequencer();
   void foxGenWaveform(int i,QString fm);
   void writeFoxQSO (QString const& msg);
+  void update_foxLogWindow_rate();
   void to_jt9(qint32 n, qint32 istart, qint32 idone);
   bool is77BitMode () const;
   void cease_auto_Tx_after_QSO ();
   Q_SLOT void ARRL_Digi_Display();
   void ARRL_Digi_Update(DecodedText dt);
   void activeWorked(QString call, QString band);
+  void read_log();
+  void refreshPileupList();
+  QString userAgent();
+  void handleVerifyMsg(int status, QDateTime ts, QString callsign, QString code, unsigned int hz, QString const &response);
+  void writeFoxTxMsgs();
+#ifdef FOX_OTP
+  QString foxOTPcode();
+#endif
 };
 
 extern int killbyname(const char* progName);
