@@ -498,6 +498,16 @@ void WideGraph::on_zero2dSlider_valueChanged(int value)               //Zero2
   }
 }
 
+void WideGraph::setSuperFox(bool b)
+{
+  ui->widePlot->setSuperFox(b);
+}
+
+void WideGraph::setSuperHound(bool b)
+{
+  ui->widePlot->setSuperHound(b);
+}
+
 void WideGraph::setTol(int n)                                         //setTol
 {
   ui->widePlot->setTol(n);
